@@ -3,7 +3,7 @@
  *
  * This file is part of mpop, a POP3 client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -31,7 +31,6 @@
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
-extern int errno;
 
 #include "gettext.h"
 #include "xalloc.h"
@@ -62,7 +61,7 @@ account_t *account_new(const char *conffile, const char *id)
     a->host = NULL;
     a->port = 0;		/* this must be set later */
     a->timeout = 0;
-    a->pipelining = 0;
+    a->pipelining = 2;
     a->delivery_method = -1;
     a->delivery_args = NULL;
     a->uidls_file = NULL;	/* this must be set later */
@@ -446,7 +445,14 @@ int check_account(account_t *acc, int retrmail, char **errstr)
     if (acc->tls_nocertcheck && acc->tls_trust_file)
     {
 	*errstr = xasprintf(
-		_("cannot use tls_trust_file with tls_cert_check turned off"));
+		_("cannot use tls_trust_file with tls_certcheck turned off"));
+	return CONF_ESYNTAX;
+    }
+    if (acc->tls && !acc->tls_trust_file && !acc->tls_nocertcheck)
+    {
+	*errstr = xasprintf(
+		_("tls requires either tls_trust_file (highly recommended) or "
+		    "a disabled tls_certcheck"));
 	return CONF_ESYNTAX;
     }
     if (retrmail && acc->delivery_method == -1)
@@ -862,6 +868,10 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    {
 		acc->pipelining = 0;
 	    }
+	    else if (strcmp(arg, "auto") == 0)
+	    {
+		acc->pipelining = 2;
+	    }
 	    else
 	    {
 		*errstr = xasprintf(
