@@ -3,7 +3,7 @@
  *
  * This file is part of mpop, a POP3 client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -32,7 +32,6 @@
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
-extern int errno;
 #include <time.h>
 #include <getopt.h>
 extern char *optarg;
@@ -1004,15 +1003,29 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
     }
     /* In theory, it is not necessary to get the capabilities again at this
      * point, because we won't use capabilities that may change between
-     * AUTHENTICATION and TRANSACTION state (see RFC 2449)
+     * AUTHENTICATION and TRANSACTION state (see RFC 2449).
      * However, as of 2005-01-17, at least pop.gmail.com violates this part of
      * the RFC by announcing the PIPELINING, UIDL, and TOP capabilities only
-     * after authentication. Since UIDL and/or TOP may be needed, we need to
-     * get the capabilities again... */
+     * after authentication. (Update 2007-03-19: now UIDL is announced before
+     * authentication, but TOP and PIPELINING still are not).
+     * This general workaround issues CAPA again if it is supported and
+     * - filtering is used and the TOP capability was not seen or
+     * - pipelining is to be set automatically and the PIPELINING capability
+     *   was not seen or
+     * - the UIDL capability was not seen.
+     * Other capabilities won't be used, so we do not care about them.
+     */
     if ((session->cap.flags & POP3_CAP_CAPA)
-	    && ((acc->filter && !(session->cap.flags & POP3_CAP_TOP))
+	    && ((acc->filter 
+		    && !(session->cap.flags & POP3_CAP_TOP))
+		|| (acc->pipelining == 2 
+		    && !(session->cap.flags & POP3_CAP_PIPELINING))
 		|| !(session->cap.flags & POP3_CAP_UIDL)))
     {
+	if (acc->pipelining == 2)
+	{
+	    session->pipelining = 2;
+	}
 	if ((e = pop3_capa(session, errstr)) != POP3_EOK)
 	{
 	    mpop_endsession(session, 0);
@@ -1765,6 +1778,10 @@ int main(int argc, char *argv[])
 		{
 		    cmdline_account->pipelining = 0;
 		}
+		else if (strcmp(optarg, "auto") == 0)
+		{
+		    cmdline_account->pipelining = 2;
+		}
 		else
 		{
 		    print_error(_("invalid argument %s for %s"), 
@@ -2096,7 +2113,7 @@ int main(int argc, char *argv[])
 	printf(_("disabled"));
 #endif
 	printf("\n\n");
-	printf(_("Copyright (C) 2006 Martin Lambers and others.\n"
+	printf(_("Copyright (C) 2007 Martin Lambers and others.\n"
 		    "This is free software.  You may redistribute copies of "
 		        "it under the terms of\n"
 		    "the GNU General Public License "
@@ -2134,8 +2151,8 @@ int main(int argc, char *argv[])
 	        "  --port=number                Set port number.\n"
 	        "  --timeout=(off|seconds)      Set/unset network timeout in "
 			"seconds.\n"
-		"  --pipelining=(on|off)        Enable/disable POP3 pipelining "
-			"for old servers.\n"
+		"  --pipelining=(auto|on|off)   Enable/disable POP3 "
+			"pipelining.\n"
 		"  --auth[=(on|method)]         Choose the authentication "
 			"method.\n"
 		"  --user=[username]            Set/unset user name for "
@@ -2331,9 +2348,10 @@ int main(int argc, char *argv[])
 		    printf(_("%d seconds\n"), account->timeout);
 		}
 	    }
-	    printf("pipelining      = %s\n"
-		    "auth            = ",
-		    account->pipelining ? _("on") : _("off"));
+	    printf("pipelining      = %s\n", 
+		    account->pipelining == 0 ? _("off") 
+		    : account->pipelining == 1 ? _("on") : _("auto"));
+	    printf("auth            = ");
 	    if (account->auth_mech[0] == '\0')
 	    {
 		printf(_("choose\n"));
