@@ -3,7 +3,7 @@
  *
  * This file is part of mpop, a POP3 client.
  *
- * Copyright (C) 2003, 2004, 2005, 2006
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -34,7 +34,6 @@
 #include <strings.h>
 #include <ctype.h>
 #include <errno.h>
-extern int errno;
 #if HAVE_SIGACTION
 # include <signal.h>
 #endif
@@ -89,8 +88,12 @@ extern int errno;
  * begin to read its answers, and refill the command pipeline when the number of
  * unanswered commands drops to PIPELINE_MIN.
  */
+#ifndef POP3_PIPELINE_MIN
 #define POP3_PIPELINE_MIN	20
+#endif
+#ifndef POP3_PIPELINE_MAX
 #define POP3_PIPELINE_MAX	100
+#endif
 
 
 /*
@@ -99,7 +102,7 @@ extern int errno;
  * see pop3.h
  */
 
-pop3_session_t *pop3_session_new(int force_pipelining,
+pop3_session_t *pop3_session_new(int pipelining,
 	const char *local_hostname, const char *local_user,
 	FILE *debug)
 {
@@ -131,7 +134,7 @@ pop3_session_t *pop3_session_new(int force_pipelining,
     tls_clear(&session->tls);
 #endif /* HAVE_TLS */
     session->cap.flags = 0;
-    session->pipelining = force_pipelining;
+    session->pipelining = pipelining;
     /* every POP3 server supports this: */
     session->cap.flags |= POP3_CAP_AUTH_USER;
     session->cap.apop_timestamp = NULL;
@@ -401,6 +404,257 @@ int pop3_send_cmd(pop3_session_t *session, char **errstr,
 }
 
 
+/*
+ * pop3_get_addr()
+ *
+ * Reads the next mail address from the given string and returns it in an
+ * allocated buffer. If no mail address is found, NULL will be returned.
+ * If a buffer is returned, the string in it will only contain the following
+ * characters: letters a-z and A-Z, digits 0-9, and any of ".@_-+/".
+ * Note that this is only a subset of what the RFCs 2821 and 2822 allow!
+ */
+char *pop3_get_addr(const char *s)
+{
+    enum states { STATE_DEFAULT, STATE_DQUOTE, 
+	STATE_BRACKETS_START, STATE_IN_BRACKETS, 
+	STATE_PARENTH_START, STATE_IN_PARENTH,
+	STATE_IN_ADDRESS, STATE_BACKQUOTE,
+        STATE_END };
+    int state = STATE_DEFAULT;
+    int oldstate = STATE_DEFAULT;
+    int backquote_savestate = STATE_DEFAULT;
+    int parentheses_depth = 0;
+    int parentheses_savestate = STATE_DEFAULT;
+    char *addr = NULL;
+    size_t addr_len = 0;
+    int forget_addr = 0;
+    int finish_addr = 0;
+    size_t bufsize = 0;
+    /* The buffer that is filled with the mail address grows by
+     * 'bufsize_step' if the remaining space becomes too small. This value must
+     * be at least 2. Wasted characters are at most (bufsize_step - 1). A value
+     * of 10 means low wasted space and a low number of realloc()s per
+     * recipient. */
+    const size_t bufsize_step = 10;
+    const char *p = s;
+ 
+    for (;;)
+    {
+	oldstate = state;
+	if (!*p)
+	{
+	    if (addr)
+		finish_addr = 1;
+	    state = STATE_END;
+	}
+	else
+	{
+	    switch (state)
+	    {
+	       	case STATE_DEFAULT:
+	    	    if (*p == '\\')
+	    	    {
+	       		backquote_savestate = state;
+	    		state = STATE_BACKQUOTE;
+	    	    }
+	    	    else if (*p == '(')
+	    	    {
+	    		parentheses_savestate = state;
+	    		state = STATE_PARENTH_START;
+	    	    }
+	    	    else if (*p == '"')
+	    	    {
+	    		if (addr)
+	    		    forget_addr = 1;
+	    		state = STATE_DQUOTE;
+	    	    }
+	    	    else if (*p == '<')
+	    	    {
+	    		if (addr)
+	    		    forget_addr = 1;
+	    		state = STATE_BRACKETS_START;
+	    	    }
+	    	    else if (*p == ' ' || *p == '\t')
+	    		; /* keep state */
+	    	    else if (*p == ':')
+	    	    {
+	    		if (addr)
+	    		    forget_addr = 1;
+	    	    }
+	    	    else if (*p == ';' || *p == ',')
+	    	    {
+	    		if (addr)
+			{
+	    		    finish_addr = 1;
+			    state = STATE_END;
+			}
+	    	    }
+	    	    else
+	    	    {
+	    		if (addr)
+	    		    forget_addr = 1;
+	    		state = STATE_IN_ADDRESS;
+	    	    }
+	    	    break;
+
+		case STATE_DQUOTE:
+	    	    if (*p == '\\')
+	    	    {
+	    		backquote_savestate = state;
+	    		state = STATE_BACKQUOTE;
+	    	    }
+	    	    else if (*p == '"')
+	    		state = STATE_DEFAULT;
+	    	    break;
+
+		case STATE_BRACKETS_START:
+	    	    if (*p == '(')
+	    	    {
+	    		parentheses_savestate = state;
+	    		state = STATE_PARENTH_START;
+	    	    }
+	    	    else if (*p == '>')
+	    		state = STATE_DEFAULT;
+	    	    else
+	    		state = STATE_IN_BRACKETS;
+	    	    break;
+
+		case STATE_IN_BRACKETS:
+	    	    if (*p == '\\')
+	    	    {
+	    		backquote_savestate = state;
+	    		state = STATE_BACKQUOTE;
+	    	    }
+	    	    else if (*p == '(')
+	    	    {
+	    		parentheses_savestate = state;
+	    		state = STATE_PARENTH_START;
+	    	    }
+	    	    else if (*p == '>')
+		    {
+			finish_addr = 1;
+	    		state = STATE_END;
+    		    }
+	    	    break;
+
+		case STATE_PARENTH_START:
+	    	    if (*p == ')')
+	    		state = parentheses_savestate;
+	    	    else
+	    	    {
+	    		parentheses_depth++;
+	    		state = STATE_IN_PARENTH;
+	    	    }
+	    	    break;
+
+		case STATE_IN_PARENTH:
+	    	    if (*p == '\\')
+	    	    {
+	    		backquote_savestate = state;
+	    		state = STATE_BACKQUOTE;
+	    	    }
+	    	    else if (*p == '(')
+	    		state = STATE_PARENTH_START;
+	    	    else if (*p == ')')
+	    	    {
+	    		parentheses_depth--;
+	    		if (parentheses_depth == 0)
+	    		    state = parentheses_savestate;
+	    	    }
+	    	    break;
+
+		case STATE_IN_ADDRESS:
+	    	    if (*p == '\\')
+	    	    {
+	    		backquote_savestate = state;
+	    		state = STATE_BACKQUOTE;
+	    	    }
+	    	    else if (*p == '"')
+	    	    {
+	    		forget_addr = 1;
+	    		state = STATE_DQUOTE;
+	    	    }
+	    	    else if (*p == '(')
+	    	    {
+	    		parentheses_savestate = state;
+	    		state = STATE_PARENTH_START;
+	    	    }
+	    	    else if (*p == '<')
+	    	    {
+	    		forget_addr = 1;
+	    		state = STATE_BRACKETS_START;
+	    	    }
+	    	    else if (*p == ' ' || *p == '\t')
+	    		state = STATE_DEFAULT;
+	    	    else if (*p == ':')
+	    	    {
+	    		forget_addr = 1;
+	    		state = STATE_DEFAULT;
+	    	    }
+	    	    else if (*p == ',' || *p == ';')
+		    {
+	    		finish_addr = 1;
+	    		state = STATE_END;
+    		    }
+	    	    break;
+
+		case STATE_BACKQUOTE:
+		    state = backquote_savestate;
+		    break;
+	    }
+	}
+
+	if (forget_addr)
+	{
+	    /* this was just junk */
+	    free(addr);
+	    addr = NULL;
+	    addr_len = 0;
+	    bufsize = 0;
+	    forget_addr = 0;
+	}
+	if (finish_addr)
+	{
+	    addr[addr_len] = '\0';
+	}
+	if (state == STATE_END)
+	{
+	    break;
+	}
+	if ((state == STATE_IN_ADDRESS || state == STATE_IN_BRACKETS)
+		&& oldstate != STATE_PARENTH_START
+		&& oldstate != STATE_IN_PARENTH)
+	{
+    	    /* Add this character to the current recipient */
+	    addr_len++;
+	    if (bufsize < addr_len + 1)
+	    {
+		bufsize += bufsize_step;
+		addr = xrealloc(addr, bufsize * sizeof(char));
+	    }
+	    /* sanitize characters */
+	    if (c_isalpha((unsigned char)*p) || c_isdigit((unsigned char)*p)
+		    || *p == '.' || *p == '@' || *p == '_' || *p == '-' 
+		    || *p == '+' || *p == '/')
+	    {
+		addr[addr_len - 1] = *p;
+	    }
+	    else if (c_isspace((unsigned char)*p))
+	    {
+		addr[addr_len - 1] = '-';
+	    }
+	    else
+	    {
+		addr[addr_len - 1] = '_';
+	    }
+	}
+	p++;
+    }
+
+    return addr;
+}
+
+
 /*
  * pop3_get_greeting()
  *
@@ -411,7 +665,7 @@ int pop3_get_greeting(pop3_session_t *session, char *greeting,
 	char **errmsg, char **errstr)
 {
     int e;
-    char *p, *q;
+    char *p, *q, *a;
     
     if ((e = pop3_get_msg(session, 0, errstr)) != POP3_EOK)
     {
@@ -429,17 +683,25 @@ int pop3_get_greeting(pop3_session_t *session, char *greeting,
 	/* 'greeting' is large enough */
 	strcpy(greeting, session->buffer + 4);
     }
-    /* search APOP timestamp */
-    if ((p = strchr(session->buffer, '<')) != NULL)
+    /* Search APOP timestamp. Make sure that it is a valid RFC822 message id as
+     * required by RFC 1939. This should make man-in-the-middle attacks as 
+     * described in CVE-2007-1558 a little bit harder. Nevertheless, APOP is 
+     * considered broken, and is never used automatically unless TLS is active.
+     */
+    a = NULL;
+    if ((p = strchr(session->buffer, '<')) != NULL	/* start of timestamp */
+	    && (q = strchr(p + 1, '>')) != NULL		/* end of timestamp */
+	    && (a = pop3_get_addr(p))			/* valid address */
+	    && strchr(a, '@')				/* has domain part */
+	    && strlen(a) + 2 == (size_t)(q - p + 1)	/* no specials */
+	    && strncmp(p + 1, a, q - p - 1) == 0)	/* no invalid chars */
     {
-	if ((q = strchr(p, '>')) != NULL)
-	{
-	    session->cap.flags |= POP3_CAP_AUTH_APOP;
-	    session->cap.apop_timestamp = xmalloc((q - p + 2) * sizeof(char));
-	    strncpy(session->cap.apop_timestamp, p, q - p + 1);
-	    session->cap.apop_timestamp[q - p + 1] = '\0';
-	}
+	session->cap.flags |= POP3_CAP_AUTH_APOP;
+	session->cap.apop_timestamp = xmalloc((q - p + 2) * sizeof(char));
+	strncpy(session->cap.apop_timestamp, p, q - p + 1);
+	session->cap.apop_timestamp[q - p + 1] = '\0';
     }
+    free(a);
 
     return POP3_EOK;
 }
@@ -617,11 +879,12 @@ int pop3_capa(pop3_session_t *session, char **errstr)
 	return POP3_EPROTO;
     }
     
-    /* Set the pipelining flag if we know whether pipelining is supported or
-     * not. Otherwise don't change it. */
-    if (session->cap.flags & POP3_CAP_CAPA) 
+    /* Automatically set the pipelining flag to off (0) or on (1) if it is not
+     * already set. */
+    if (session->pipelining == 2)
     {
-	if (session->cap.flags & POP3_CAP_PIPELINING)
+	if (session->cap.flags & POP3_CAP_CAPA 
+		&& session->cap.flags & POP3_CAP_PIPELINING)
 	{
 	    session->pipelining = 1;
 	}
@@ -806,15 +1069,10 @@ int pop3_uidl_check_uid(const char *uid)
     const char *p = uid;
     
     /* According to RFC 1939, a valid UID must consist of one to 70 characters
-     * in the range 0x21 to 0x7e. We allow longer UIDs, spaces inside UIDs (but
-     * no space at the beginning of an UID), and non-ASCII characters in UIDs
-     * as long as they are not control characters.
+     * in the range 0x21 to 0x7e. We allow longer UIDs, spaces inside UIDs, and
+     * non-ASCII characters in UIDs as long as they are not control characters.
      * I know of one case where the POP3 server uses non-ASCII characters in 
      * UIDs. I don't know if any server needs the other extensions, though. */
-    if (*p == ' ')
-    {
-	return 0;
-    }
     while (*p != '\0')
     {
 	if (c_iscntrl((unsigned char)*p))
@@ -890,12 +1148,23 @@ int pop3_uidl(pop3_session_t *session, char **errmsg, char **errstr)
 	    if (p == session->buffer || *p != ' ' 
 		    || (n == LONG_MAX && errno == ERANGE)
 		    || n < 1 || n > session->total_number
-		    || session->msg_uid[n - 1]
-		    || !pop3_uidl_check_uid(p + 1))
+		    || session->msg_uid[n - 1])
 	    {
 		goto invalid_reply;
 	    }
-	    session->msg_uid[n - 1] = xstrdup(p + 1);
+	    /* Allow more than one space between the number and the UID, even
+	     * though RFC 1939 says it must be exactly one. Needed for the
+	     * "Maillennium V05.00c++" POP3 server used by Comcast.net as of
+	     * 2007-01-29. */
+	    while (*p == ' ')
+	    {
+		p++;
+	    }
+	    if (!pop3_uidl_check_uid(p))
+	    {
+		goto invalid_reply;
+	    }
+	    session->msg_uid[n - 1] = xstrdup(p);
 	    insert_sorted(session->uids_sorted, i, session->msg_uid, n - 1);
 	}
     }
@@ -1226,257 +1495,6 @@ int pop3_pipe(pop3_session_t *session,
 }
 
 
-/*
- * pop3_get_addr()
- *
- * Reads the next mail address from the given string and returns it in an
- * allocated buffer. If no mail address is found, NULL will be returned.
- * If a buffer is returned, the string in it will only contain the following
- * characters: letters a-z and A-Z, digits 0-9, and any of ".@_-+/".
- * Note that this is only a subset of what the RFCs 2821 and 2822 allow!
- */
-char *pop3_get_addr(const char *s)
-{
-    enum states { STATE_DEFAULT, STATE_DQUOTE, 
-	STATE_BRACKETS_START, STATE_IN_BRACKETS, 
-	STATE_PARENTH_START, STATE_IN_PARENTH,
-	STATE_IN_ADDRESS, STATE_BACKQUOTE,
-        STATE_END };
-    int state = STATE_DEFAULT;
-    int oldstate = STATE_DEFAULT;
-    int backquote_savestate = STATE_DEFAULT;
-    int parentheses_depth = 0;
-    int parentheses_savestate = STATE_DEFAULT;
-    char *addr = NULL;
-    size_t addr_len = 0;
-    int forget_addr = 0;
-    int finish_addr = 0;
-    size_t bufsize = 0;
-    /* The buffer that is filled with the mail address grows by
-     * 'bufsize_step' if the remaining space becomes too small. This value must
-     * be at least 2. Wasted characters are at most (bufsize_step - 1). A value
-     * of 10 means low wasted space and a low number of realloc()s per
-     * recipient. */
-    const size_t bufsize_step = 10;
-    const char *p = s;
- 
-    for (;;)
-    {
-	oldstate = state;
-	if (!*p)
-	{
-	    if (addr)
-		finish_addr = 1;
-	    state = STATE_END;
-	}
-	else
-	{
-	    switch (state)
-	    {
-	       	case STATE_DEFAULT:
-	    	    if (*p == '\\')
-	    	    {
-	       		backquote_savestate = state;
-	    		state = STATE_BACKQUOTE;
-	    	    }
-	    	    else if (*p == '(')
-	    	    {
-	    		parentheses_savestate = state;
-	    		state = STATE_PARENTH_START;
-	    	    }
-	    	    else if (*p == '"')
-	    	    {
-	    		if (addr)
-	    		    forget_addr = 1;
-	    		state = STATE_DQUOTE;
-	    	    }
-	    	    else if (*p == '<')
-	    	    {
-	    		if (addr)
-	    		    forget_addr = 1;
-	    		state = STATE_BRACKETS_START;
-	    	    }
-	    	    else if (*p == ' ' || *p == '\t')
-	    		; /* keep state */
-	    	    else if (*p == ':')
-	    	    {
-	    		if (addr)
-	    		    forget_addr = 1;
-	    	    }
-	    	    else if (*p == ';' || *p == ',')
-	    	    {
-	    		if (addr)
-			{
-	    		    finish_addr = 1;
-			    state = STATE_END;
-			}
-	    	    }
-	    	    else
-	    	    {
-	    		if (addr)
-	    		    forget_addr = 1;
-	    		state = STATE_IN_ADDRESS;
-	    	    }
-	    	    break;
-
-		case STATE_DQUOTE:
-	    	    if (*p == '\\')
-	    	    {
-	    		backquote_savestate = state;
-	    		state = STATE_BACKQUOTE;
-	    	    }
-	    	    else if (*p == '"')
-	    		state = STATE_DEFAULT;
-	    	    break;
-
-		case STATE_BRACKETS_START:
-	    	    if (*p == '(')
-	    	    {
-	    		parentheses_savestate = state;
-	    		state = STATE_PARENTH_START;
-	    	    }
-	    	    else if (*p == '>')
-	    		state = STATE_DEFAULT;
-	    	    else
-	    		state = STATE_IN_BRACKETS;
-	    	    break;
-
-		case STATE_IN_BRACKETS:
-	    	    if (*p == '\\')
-	    	    {
-	    		backquote_savestate = state;
-	    		state = STATE_BACKQUOTE;
-	    	    }
-	    	    else if (*p == '(')
-	    	    {
-	    		parentheses_savestate = state;
-	    		state = STATE_PARENTH_START;
-	    	    }
-	    	    else if (*p == '>')
-		    {
-			finish_addr = 1;
-	    		state = STATE_END;
-    		    }
-	    	    break;
-
-		case STATE_PARENTH_START:
-	    	    if (*p == ')')
-	    		state = parentheses_savestate;
-	    	    else
-	    	    {
-	    		parentheses_depth++;
-	    		state = STATE_IN_PARENTH;
-	    	    }
-	    	    break;
-
-		case STATE_IN_PARENTH:
-	    	    if (*p == '\\')
-	    	    {
-	    		backquote_savestate = state;
-	    		state = STATE_BACKQUOTE;
-	    	    }
-	    	    else if (*p == '(')
-	    		state = STATE_PARENTH_START;
-	    	    else if (*p == ')')
-	    	    {
-	    		parentheses_depth--;
-	    		if (parentheses_depth == 0)
-	    		    state = parentheses_savestate;
-	    	    }
-	    	    break;
-
-		case STATE_IN_ADDRESS:
-	    	    if (*p == '\\')
-	    	    {
-	    		backquote_savestate = state;
-	    		state = STATE_BACKQUOTE;
-	    	    }
-	    	    else if (*p == '"')
-	    	    {
-	    		forget_addr = 1;
-	    		state = STATE_DQUOTE;
-	    	    }
-	    	    else if (*p == '(')
-	    	    {
-	    		parentheses_savestate = state;
-	    		state = STATE_PARENTH_START;
-	    	    }
-	    	    else if (*p == '<')
-	    	    {
-	    		forget_addr = 1;
-	    		state = STATE_BRACKETS_START;
-	    	    }
-	    	    else if (*p == ' ' || *p == '\t')
-	    		state = STATE_DEFAULT;
-	    	    else if (*p == ':')
-	    	    {
-	    		forget_addr = 1;
-	    		state = STATE_DEFAULT;
-	    	    }
-	    	    else if (*p == ',' || *p == ';')
-		    {
-	    		finish_addr = 1;
-	    		state = STATE_END;
-    		    }
-	    	    break;
-
-		case STATE_BACKQUOTE:
-		    state = backquote_savestate;
-		    break;
-	    }
-	}
-
-	if (forget_addr)
-	{
-	    /* this was just junk */
-	    free(addr);
-	    addr = NULL;
-	    addr_len = 0;
-	    bufsize = 0;
-	    forget_addr = 0;
-	}
-	if (finish_addr)
-	{
-	    addr[addr_len] = '\0';
-	}
-	if (state == STATE_END)
-	{
-	    break;
-	}
-	if ((state == STATE_IN_ADDRESS || state == STATE_IN_BRACKETS)
-		&& oldstate != STATE_PARENTH_START
-		&& oldstate != STATE_IN_PARENTH)
-	{
-    	    /* Add this character to the current recipient */
-	    addr_len++;
-	    if (bufsize < addr_len + 1)
-	    {
-		bufsize += bufsize_step;
-		addr = xrealloc(addr, bufsize * sizeof(char));
-	    }
-	    /* sanitize characters */
-	    if (c_isalpha((unsigned char)*p) || c_isdigit((unsigned char)*p)
-		    || *p == '.' || *p == '@' || *p == '_' || *p == '-' 
-		    || *p == '+' || *p == '/')
-	    {
-		addr[addr_len - 1] = *p;
-	    }
-	    else if (c_isspace((unsigned char)*p))
-	    {
-		addr[addr_len - 1] = '-';
-	    }
-	    else
-	    {
-		addr[addr_len - 1] = '_';
-	    }
-	}
-	p++;
-    }
-
-    return addr;
-}
-
-
 /* 
  * pop3_retr_get_from_addr()
  *
@@ -2667,19 +2685,14 @@ int pop3_auth(pop3_session_t *session,
 	{
 	    auth_mech = "CRAM-MD5";
 	}
-	else if (session->cap.flags & POP3_CAP_AUTH_APOP)
-	{
-	    auth_mech = "APOP";
-	}
-	else if (gsasl_client_support_p(ctx, "NTLM") 
-		&& (session->cap.flags & POP3_CAP_AUTH_NTLM))
-	{
-	    auth_mech = "NTLM";
-	}
 #ifdef HAVE_TLS
 	else if (tls_is_active(&session->tls))
 	{
-	    if (gsasl_client_support_p(ctx, "PLAIN") 
+	    if (session->cap.flags & POP3_CAP_AUTH_APOP)
+	    {
+		auth_mech = "APOP";
+	    }
+	    else if (gsasl_client_support_p(ctx, "PLAIN") 
 		    && (session->cap.flags & POP3_CAP_AUTH_PLAIN))
 	    {
 		auth_mech = "PLAIN";
@@ -2693,6 +2706,11 @@ int pop3_auth(pop3_session_t *session,
 	    {
 		auth_mech = "LOGIN";
 	    }
+	    else if (gsasl_client_support_p(ctx, "NTLM") 
+		    && (session->cap.flags & POP3_CAP_AUTH_NTLM))
+	    {
+		auth_mech = "NTLM";
+	    }
 	}
 #endif /* HAVE_TLS */
     }
